import graphviz            # pip install python-graphviz
import pydotplus as pdp    # pip install pydotplus

import xml.etree.ElementTree as ET
import sys


def get_xml_nodes_ids_types(root):
    return [(n.attrib['id'], n.attrib['type']) for n in root.findall('./node')]


def get_xml_edges_from_to(root):
    return [
        (e.attrib['from'], e.attrib['to'])
        for e in root.findall('./edge')
    ]


def xml2networkx(root, labels_path=None):
    nodes = get_xml_nodes_ids_types(root)
    edges = get_xml_edges_from_to(root)
    G = graphviz.Graph(format='gv', engine='sfdp', filename='mygraph.gv')
    G.graph_attr['smoothing'] = 'triangle'
    G.graph_attr['start'] = '1'
    G.graph_attr['overlap'] = 'prism'
    G.graph_attr['overlap_scaling'] = '0'
    G.graph_attr['mode'] = 'ipsep'
    G.graph_attr['sep'] = '+8'

    # exclude isolated nodes
    #all_edges = list(zip(*edges))
    #all_nodes_in_edges = all_edges[0] + all_edges[1]

    for n in nodes:
        if n[1] in ['rct']:
            shape = 'rectangle'
        elif n[1] in ['species', 'metabolite']:
            shape = 'ellipse'
        else:
            shape = 'none'

        image = None
        margin = None
        label = None
        width = None
        height = None
        if labels_path:
            shape = 'rectangle'
            image= f'{labels_path}/{n[0]}.png'
            margin = '0'
            width = '0'
            height = '0'
            label = ''

        if True: #n[0] in all_nodes_in_edges:
            G.node(
                n[0],
                image=image,
                label=label,
                shape=shape,
                width=width,
                height=height,
                margin=margin,
                #imagescale='false',
                group='one'
            )

    for e in edges:
        G.edge(*e, constraint='false', len='0.1')
    return G


def set_positions(positions, root, scale):
    for nodeid, pos in positions.items():
        nodes = root.findall(f".//node[@id='{nodeid}']")
        assert len(nodes) == 1, 'Wrong or multiple IDs'
        node = nodes[0]
        node.set('x', str(pos[0] * scale))
        node.set('y', str(pos[1] * scale))


def layout(xmlfile_in, xmlfile_out, scale, labels_path=None, view=False):
    tree = ET.parse(xmlfile_in)
    root = tree.getroot()

    G = xml2networkx(root, labels_path=labels_path)
    GV = pdp.graphviz.graph_from_dot_data(G.pipe())
    xmlnode_list = [n[0] for n in get_xml_nodes_ids_types(root)]
    positions = {
        n.get_name():tuple(map(float, n.get_pos()[1:-1].split(',')))
        for n in GV.get_node_list()
        if n.get_name() in xmlnode_list
    }

    set_positions(positions, root, scale)
    tree.write(xmlfile_out, encoding='unicode')

    GC = G.copy()
    GC.format = 'pdf'
    GC.save()
    if view:
        GC.view()


if __name__ == '__main__':
    XMLFILE_IN = None
    labels_path = None
    scale = 0.05
    if len(sys.argv) in [3, 4]:
        XMLFILE_IN = sys.argv[1]
        XMLFILE_OUT = sys.argv[2]
        if len(sys.argv) == 4:
            scale = float(sys.argv[3])
    elif len(sys.argv) in [5, 6] and sys.argv[1] == '--path-to-labels':
        labels_path = sys.argv[2]
        XMLFILE_IN = sys.argv[3]
        XMLFILE_OUT = sys.argv[4]
        if len(sys.argv) == 6:
            scale = float(sys.argv[5])
    else:
        print(f"""
    Usage:
        {sys.argv[0]} [ --path-to-labels some_path ]   xmlfile_in   xmlfile_out   [ scale ]

    The option --path-to-labels specifies the folder where the labels
    of the nodes in PNG format may be found.

    The optional parameter scale applies to the x, y coordinates written
    in the output file. The default value is 0.05.
    """
        )
        
    if XMLFILE_IN:
       layout(XMLFILE_IN, XMLFILE_OUT, scale, labels_path, view=False)

