import graphviz            # pip install python-graphviz
import pydotplus as pdp    # pip install pydotplus

import xml.etree.ElementTree as ET
import sys


def get_xml_nodes_ids_types(root):
    return [(n.attrib['id'], n.attrib['type']) for n in root.findall('./node')]


def get_xml_edges_from_to(root):
    return [
        (e.attrib['from'], e.attrib['to'])
        for e in root.findall('./edge')
    ]


def xml2networkx(root):
    nodes = get_xml_nodes_ids_types(root)
    edges = get_xml_edges_from_to(root)
    G = graphviz.Digraph(format='gv', engine='sfdp', filename='mygraph.gv')
    G.graph_attr['smoothing'] = 'triangle'
    G.graph_attr['start'] = '1'
    G.graph_attr['overlap'] = 'prism'

    # exclude isolated nodes
    #all_edges = list(zip(*edges))
    #all_nodes_in_edges = all_edges[0] + all_edges[1]

    for n in nodes:
        if n[1] in ['rct']:
            shape = 'rectangle'
        elif n[1] in ['species', 'metabolite']:
            shape = 'ellipse'
        else:
            shape = 'none'
        if True: #n[0] in all_nodes_in_edges:
            G.node(
                n[0],
                #image='latex/0.svg',
                #label='',
                shape=shape,
                #width='0',
                #height='0',
                #margin='0',
                #imagescale='false'
            )

    for e in edges:
        G.edge(*e)
    return G


def set_positions(positions, root, scale):
    for nodeid, pos in positions.items():
        nodes = root.findall(f".//node[@id='{nodeid}']")
        assert len(nodes) == 1, 'Wrong or multiple IDs'
        node = nodes[0]
        node.set('x', str(pos[0] * scale))
        node.set('y', str(pos[1] * scale))


def layout(xmlfile_in, xmlfile_out, scale, view=False):
    tree = ET.parse(xmlfile_in)
    root = tree.getroot()

    G = xml2networkx(root)
    GV = pdp.graphviz.graph_from_dot_data(G.pipe())
    xmlnode_list = [n[0] for n in get_xml_nodes_ids_types(root)]
    positions = {
        n.get_name():tuple(map(float, n.get_pos()[1:-1].split(',')))
        for n in GV.get_node_list()
        if n.get_name() in xmlnode_list
    }

    set_positions(positions, root, scale)
    tree.write(xmlfile_out, encoding='unicode')

    GC = G.copy()
    GC.format = 'pdf'
    GC.save()
    if view:
        GC.view()


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print(f"""
    Usage:
        {sys.argv[0]}   xmlfile_in   xmlfile_out   [ scale ]

    The optional parameter scale applies to the x, y coordinates written
    in the output file. The default value is 0.05.
    """
        )
    else:
        if len(sys.argv) == 3:
            scale = 0.05
        else:
            scale = float(sys.argv[3])
        layout(sys.argv[1], sys.argv[2], scale)
